package de.akquinet.jbosscc.examples.eap.bean;

import de.akquinet.jbosscc.examples.eap.ejb.SecurityService;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.security.auth.Subject;
import java.security.AccessController;
import java.time.LocalDateTime;
import java.util.logging.Logger;

public class BatchJobImpl implements BatchJob {
    private static final long serialVersionUID = 1L;

    private static final Logger LOGGER = Logger.getLogger(BatchJobImpl.class.getName());

    private final String name;
    private final SecurityService securityService;
    private final LocalDateTime creationTime = LocalDateTime.now();
    private boolean running;

    public BatchJobImpl(final String name, final SecurityService securityService) {
        this.name = name;
        this.securityService = securityService;

        log("CREATED");
    }


    @Override
    public String call() throws Exception {
        log("START");

        work();

        log("DONE");
        return name;
    }

    public void work() throws Exception {
        logSubject();

        running = true;

        for (int i = 0; i < 10; i++) {
            Thread.sleep(2000L);
            log("RUNNING " + i);
        }
    }

    private void logSubject() {
        final Subject subject = Subject.getSubject(AccessController.getContext());

        if (subject != null) {
            log("Subject is " + subject.getPrincipals());
        } else {
            log("Subject is NULL");
        }
    }

    @Override
    public boolean isRunning() {
        return running;
    }

    private void log(final String msg) {
        final String message = name + ":[" + Thread.currentThread().getName() + "](" + creationTime + "): " + msg;
        final String principalInfo = securityService.getPrincipal();

        LOGGER.info(message);
        LOGGER.info("Principal:" + principalInfo);

        final FacesContext facesContext = FacesContext.getCurrentInstance();

        // Does not work after Thread.sleep anymore. Instance is thread-local and bound to the request
        if (facesContext != null) {
            facesContext.addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO, message, null));
        } else {
            LOGGER.warning("Faces context not found");
        }
    }

    @Override
    public int hashCode() {
        return (name + "@" + creationTime).hashCode();
    }


    @Override
    public boolean equals(final Object o) {
        if (o == null || getClass() != o.getClass()) {
            return false;
        }

        final BatchJobImpl that = (BatchJobImpl) o;

        return this.name.equals(that.name) && this.creationTime.equals(that.creationTime);
    }

    @Override
    public String toString() {
        return name + "@" + creationTime +
            (isRunning() ? " running" : "not running") +
            ", started by: " + securityService.getPrincipal();
    }

    @Override
    public String getName() {
        return name;
    }
}
