package de.akquinet.jbosscc.examples.eap.ejb;

import de.akquinet.jbosscc.examples.eap.Resources;
import de.akquinet.jbosscc.examples.eap.bean.BatchJob;
import de.akquinet.jbosscc.examples.eap.bean.BatchJobImpl;
import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.ejb.Singleton;
import javax.enterprise.concurrent.ContextService;
import javax.enterprise.concurrent.ManagedExecutorService;
import javax.enterprise.concurrent.ManagedTask;
import javax.faces.application.FacesMessage;
import javax.inject.Inject;
import javax.jms.JMSContext;
import javax.jms.JMSProducer;
import javax.jms.Message;
import javax.jms.Queue;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.stream.Collectors;

@Singleton
public class BatchService {
    private static final Logger LOGGER = Logger.getLogger(BatchService.class.getName());

    @Resource(mappedName = Resources.Queue)
    private Queue queue;

    @Inject
    private JMSContext jmsContext;

    @Resource
    private ContextService contextService;

    @Resource
    private ManagedExecutorService executorService;

    @EJB
    private SecurityService securityService;

    private final Map<BatchJob, Future<String>> batches = new LinkedHashMap<>();

    public void submitBatch(final String name) {
        final BatchJob proxy = contextService.createContextualProxy(new BatchJobImpl(name, securityService), BatchJob.class);
        final Future<String> future = executorService.submit(proxy);

        batches.put(proxy, future);
    }

    public List<String> listJobs() {
        return batches.entrySet().stream().map(entry -> toString(entry.getKey(), entry.getValue())).collect(Collectors.toList());
    }

    private static String toString(final BatchJob job, final Future<String> future) {
        return job.toString() + " -> " + toString(future);
    }

    private static String toString(final Future<String> future) {
        String status = future.isDone() ? "FINISHED" : "PENDING";
        String result = "<?>";

        if (future.isDone()) {

            try {
                result = future.get();
            } catch (InterruptedException e) {
                status = "INTERRUPTED";
                result = e.toString();
            } catch (ExecutionException e) {
                status = "ABORTED";
                Throwable exception = e.getCause();
                for (; exception.getCause() != null; exception = exception.getCause()) {

                }

                LOGGER.log(Level.SEVERE, "Running job", e.getCause());
                result = exception.toString() + ": See log file for details";
            }
        }

        return status + "=" + result;
    }

    public void submitBatchToJms(final String name) {
        final Map<String, String> execProps = new HashMap<>();
        execProps.put(ManagedTask.TRANSACTION, ManagedTask.USE_TRANSACTION_OF_EXECUTION_THREAD);

        final BatchJob proxy = contextService.createContextualProxy(new BatchJobImpl(name, securityService),
            execProps,BatchJob.class);

        // send
        Message message = jmsContext.createObjectMessage(proxy);
        JMSProducer jmsProducer = jmsContext.createProducer();
        jmsProducer.send(queue, message);
    }
}
