package de.akquinet.jbosscc.examples.eap.ejb;

import org.jboss.ejb3.annotation.SecurityDomain;
import javax.annotation.Resource;
import javax.annotation.security.RolesAllowed;
import javax.ejb.SessionContext;
import javax.ejb.Stateless;
import javax.naming.Context;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import java.security.Principal;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

@Stateless
@RolesAllowed({"user", "admin"})
@SecurityDomain("jumble-domain")
public class SecurityService {
    private static final Logger LOGGER = Logger.getLogger(SecurityService.class.getName());

    @Resource
    private SessionContext ctx;

    public String getPrincipal() {
        final Principal principal = ctx.getCallerPrincipal();

//        logEnvironment();

        return principal.getName();
    }

    private void logEnvironment() {
        final Map<String, Object> contextData = ctx.getContextData();

        LOGGER.info("Context data: " + contextData);

        try {
            final Context namingContext = (Context) ctx.lookup("java:comp/env");
            final NamingEnumeration<NameClassPair> enumeration = namingContext.list("");

            while (enumeration.hasMoreElements()) {
                final NameClassPair nameClassPair = enumeration.nextElement();
                LOGGER.info(namingContext.lookup(nameClassPair.getName()).toString());
            }
        } catch (NamingException e) {
            LOGGER.log(Level.WARNING, "log environment", e);
        }
    }
}
